import time
import numpy as npy
import scipy.special
import cv2
import imutils
from imutils.video import VideoStream

epochs = 10

# training_data_file=open("/home/pi/DATA/MNIST/mnist_train_100.csv", 'r')
training_data_file=open("/home/pi/DATA/MNIST/mnist_train.csv", 'r')
training_data_list=training_data_file.readlines()
training_data_file.close()
print("number of training datasets loaded:",end=" ")
print(len(training_data_list))

class neuralNetwork:
    def __init__(self,IN,HN,ON,LR):
        self.ino=IN; self.hno=HN; self.ono=ON
        self.wih=npy.random.normal(0.0,pow(self.ino,-0.5),(self.hno,self.ino))
        self.who=npy.random.normal(0.0,pow(self.hno,-0.5),(self.ono,self.hno))
        self.lr=LR
        self.activation_function=lambda x:scipy.special.expit(x)
        pass
    def train(self,inputs_list,targets_list):
        inputs=npy.array(inputs_list,ndmin=2).T
        targets=npy.array(targets_list,ndmin=2).T
        hidden_inputs=npy.dot(self.wih,inputs)
        hidden_outputs=self.activation_function(hidden_inputs)
        final_inputs=npy.dot(self.who,hidden_outputs)
        final_outputs=self.activation_function(final_inputs)
        output_errors=targets-final_outputs
        hidden_errors=npy.dot(self.who.T,output_errors)
        self.who+=self.lr*npy.dot((output_errors*final_outputs*(1.0-final_outputs)),npy.transpose(hidden_outputs))
        self.wih+=self.lr*npy.dot((hidden_errors*hidden_outputs*(1.0-hidden_outputs)),npy.transpose(inputs))
        pass
    def query(self,inputs_list):
        inputs=npy.array(inputs_list,ndmin=2).T
        hidden_inputs=npy.dot(self.wih,inputs)
        hidden_outputs=self.activation_function(hidden_inputs)
        final_inputs=npy.dot(self.who,hidden_outputs)
        final_outputs=self.activation_function(final_inputs)
        return final_outputs

MyNet=neuralNetwork(784,200,10,.1) # input_nodes, hidden_nodes, output_nodes, learning_rate

start_proc = time.process_time()
for epoch in range(epochs):
    print("epoch #:",epoch,end="")
    for record in training_data_list:
        all_values = record.split(',')  
        inputs = (npy.asfarray(all_values[1:]) / 255.0 * 0.99) + 0.01
        targets = npy.zeros(10) + 0.01
        targets[int(all_values[0])] = 0.99
        MyNet.train(inputs, targets)
        end_proc = time.process_time()
    print(' completed - time: {:5.3f}s'.format(end_proc-start_proc))
end_proc = time.process_time()
print('training time: {:5.3f}s'.format(end_proc-start_proc))
vs=VideoStream(usePiCamera=["picamera"]).start()
time.sleep(1.0)

while True:
    frame=imutils.resize(vs.read(), width=400)
    im_gray=cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    (thresh, im_bw)=cv2.threshold(im_gray, 128, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)
    img_resized=cv2.resize(im_bw,(28,28))
    cv2.imshow("press 'q' to quit", img_resized)
    img_data=((255.0-img_resized.reshape(784))/255.0*0.99)+0.01
    inputs = img_data
    outputs = MyNet.query(inputs)
    label = npy.argmax(outputs)
    print("MyNet reads:",label)
    key = cv2.waitKey(1) & 0xFF
    if key == ord("q"):
        break
    time.sleep(0.1)
print('bye...')
cv2.destroyAllWindows()
vs.stop()



